@@ -39,6 +39,7 @@ QTEST_KDEMAIN_CORE( KUrlTest )
 #include <QtCore/QMap>
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h> // gethostname
 
 //QCOMPARE cannot be used to strictly check for empty or null QString as it treats QString("") == QString()
 #define QSTREMPTY(_str) QVERIFY(!_str.isNull() && _str.isEmpty())
@@ -73,6 +74,9 @@ void KUrlTest::testEmptyURL()
   fileURL = "file:///";
   QVERIFY( !fileURL.isEmpty() );
 
+  fileURL = QLatin1String("FILE:///");
+  QVERIFY( fileURL.isLocalFile() );
+
   KUrl udir;
   QCOMPARE( udir.url(), QString() );
   QVERIFY( udir.isEmpty() );
@@ -107,6 +111,13 @@ void KUrlTest::testSetQuery()
   QCOMPARE( url1.query(), QString("?") );
   url1.setQuery( QString() );
   QCOMPARE( url1.query(), QString() );
+
+  url1.setQuery("4=2+2");
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi?4=2+2"));
+  QCOMPARE( url1.query(), QString("?4=2+2"));
+  url1.setQuery("4=2%2B2");
+  QCOMPARE( url1.url(), QString("http://www.kde.org/foo.cgi?4=2%2B2")); // %2B should not be decoded to +
+  QCOMPARE( url1.query(), QString("?4=2%2B2"));
 }
 
 void KUrlTest::testEmptyNullReference()
@@ -129,7 +140,7 @@ void KUrlTest::testEmptyNullReference()
 void KUrlTest::testSetRef()
 {
   KUrl url1 = KUrl( QByteArray( "http://www.kde.org/foo.cgi#foo=bar" ) );
-  QCOMPARE( url1.ref(), QString("foo%3Dbar" ) ); // KDE3 difference: was foo=bar
+  QCOMPARE( url1.ref(), QString("foo=bar" ) );
 #if 0// ditto (TODO)
   url1.setRef( "toto=titi&kde=rocks" );
   QCOMPARE( url1.ref(), QString("toto=titi&kde=rocks" ) );
@@ -198,8 +209,17 @@ void KUrlTest::testIsLocalFile()
   KUrl local_file_2("file://www.kde.org/my/file");
   QVERIFY( !local_file_2.isLocalFile() );
 
+  // exactly like in kurl.cpp, getenv("HOSTNAME") can be different...
+  char hostname[ 256 ];
+  hostname[ 0 ] = '\0';
+  if (!gethostname( hostname, 255 ))
+     hostname[sizeof(hostname)-1] = '\0';
+
+  for(char *p = hostname; *p; p++)
+     *p = tolower(*p);
+
   KUrl local_file_3;
-  local_file_3.setHost(getenv("HOSTNAME"));
+  local_file_3.setHost(QString::fromLatin1(hostname));
   local_file_3.setPath("/my/file");
   //qDebug("URL=%s\n", qPrintable( local_file_3.url() ));
   QVERIFY( local_file_3.isLocalFile() );
@@ -221,6 +241,9 @@ void KUrlTest::testIsLocalFile()
   local_file_5a.setPath("c:/foo?bar");
   QCOMPARE( local_file_5a.url(), QString("file:///c:/foo%3Fbar") );
 #endif
+
+  KUrl scummos("inf://localhost/test.txt");
+  QVERIFY(!scummos.isLocalFile());
 }
 
 void KUrlTest::testSimpleMethods() // to test parsing, mostly
@@ -238,6 +261,29 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QCOMPARE(ulong.host(),QString("swww.gad.de") );
   QCOMPARE(ulong.path(),QString("/servlet/CookieAccepted") );
 
+  // RFC 3986 section 3.5 says:
+  // fragment = *( pchar / "/" / "?" )
+  // where:
+  // pchar = unreserved / pct-encoded / sub-delims / ":" / "@"
+  //         unreserved  = ALPHA / DIGIT / "-" / "." / "_" / "~"
+  //         pct-encoded = "%" HEXDIG HEXDIG
+  //         sub-delims  = sub-delims  = "!" / "$" / "&" / "'" / "(" / ")" /  "*" / "+" / "," / ";" / "="
+  KUrl url0;
+  url0 = QString("http://www.kde.org#/?:@-._~!$&'()*+,;="); // These fragment's chars should not be %-encoded
+  QCOMPARE( url0.ref(),     QString("/?:@-._~!$&'()*+,;=") );
+  QCOMPARE( url0.htmlRef(), QString("/?:@-._~!$&'()*+,;=") );
+  QCOMPARE( url0.url(), QString("http://www.kde.org#/?:@-._~!$&'()*+,;=") );
+
+  url0 = QString("http://www.kde.org#/?:@-._~!$&'()*+,;=%A");
+  QCOMPARE( url0.ref(),     QString("/?:@-._~!$&'()*+,;=%25A") );
+  QCOMPARE( url0.htmlRef(), QString("/?:@-._~!$&'()*+,;=%A") );
+  QCOMPARE( url0.url(), QString("http://www.kde.org#/?:@-._~!$&'()*+,;=%25A") );
+
+  url0 = "http://www.kde.org/foo.cgi?4=2+2";
+  QCOMPARE( url0.url(), QString("http://www.kde.org/foo.cgi?4=2+2") );
+  url0 = "http://www.kde.org/foo.cgi?4=2%2B2";
+  QCOMPARE( url0.url(), QString("http://www.kde.org/foo.cgi?4=2%2B2") );  // %2B should not be decoded to +
+
   KUrl fileURL( "file:///home/dfaure/myfile" );
   QCOMPARE( fileURL.url(), QString("file:///home/dfaure/myfile") );
   QCOMPARE( fileURL.path(), QString("/home/dfaure/myfile") );
@@ -290,16 +336,23 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QCOMPARE( url1.encodedHtmlRef(), QString("%23") );
   QCOMPARE( url1.htmlRef(), QString("#") );
 
-#if 0 // TODO
   url1 = KUrl(url1, "#%6a");
-  QCOMPARE( url1.url(), QString("file:///home/dfaure/my#%6a") );
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my#j") );
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
   QCOMPARE( url1.ref(), QString("j") );
-  QCOMPARE( url1.encodedHtmlRef().toLower(), QString("%6a") );
+  QCOMPARE( url1.encodedHtmlRef(), QString("j") );
   QCOMPARE( url1.htmlRef(), QString("j") );
-#endif
+
+  url1 = KUrl(url1, "#");
+  QCOMPARE( url1.url(), QString("file:///home/dfaure/my#") );
+  QVERIFY( url1.hasRef() );
+  QVERIFY( url1.hasHTMLRef() );
+  QVERIFY( !url1.hasSubUrl() );
+  QSTREMPTY(url1.ref());
+  QSTREMPTY(url1.encodedHtmlRef());
+  QSTREMPTY(url1.htmlRef());
 
   u1 = "file:///home/dfaure/my#myref";
   url1 = u1;
@@ -316,7 +369,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
-  QCOMPARE( url1.ref(), QString("QObject%3A%3Aconnect") );
+  QCOMPARE( url1.ref(), QString("QObject::connect") );
   QCOMPARE( url1.htmlRef(), QString("QObject::connect") );
   QCOMPARE( url1.upUrl().url(), QString("file:///opt/kde2/qt2/doc/html/") );
 
@@ -326,7 +379,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
-  QCOMPARE( url1.ref(), QString("QObject%3A%3Aconnect") );
+  QCOMPARE( url1.ref(), QString("QObject::connect") );
   QCOMPARE( url1.htmlRef(), QString("QObject::connect") );
   QCOMPARE( url1.upUrl().url(), QString("file:///opt/kde2/qt2/doc/html/") );
 
@@ -336,7 +389,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QVERIFY( url1.hasRef() );
   QVERIFY( url1.hasHTMLRef() );
   QVERIFY( !url1.hasSubUrl() );
-  QCOMPARE( url1.ref(), QString("QObject%3Aconnect") );
+  QCOMPARE( url1.ref(), QString("QObject:connect") );
   QCOMPARE( url1.htmlRef(), QString("QObject:connect") );
   QCOMPARE( url1.upUrl().url(), QString("file:///opt/kde2/qt2/doc/html/") );
 
@@ -532,7 +585,7 @@ void KUrlTest::testURLsWithoutPath()
   QCOMPARE( waba1.host(), QString( "a" ) );
   QCOMPARE( waba1.port(), 389 );
   QCOMPARE( waba1.path(), QString( "" ) );
-  QCOMPARE( waba1.ref(), QString( "b%3Dc" ) ); // was b=c with KDE3, but the docu says encoded, so encoding the = is ok
+  QCOMPARE( waba1.ref(), QString( "b=c" ) );
   QCOMPARE( waba1.htmlRef(), QString( "b=c" ) );
   QCOMPARE( waba1.query(), QString() );
 
@@ -553,7 +606,7 @@ void KUrlTest::testPathAndQuery()
 #else
   // So we treat it as part of the fragment
   KUrl tobi0("http://some.host.net/path/to/file#foo?bar");
-  QCOMPARE( tobi0.ref(), QString("foo%3Fbar") );
+  QCOMPARE( tobi0.ref(), QString("foo?bar") );
   QCOMPARE( tobi0.query(), QString() );
   QCOMPARE( tobi0.prettyUrl(), QString("http://some.host.net/path/to/file#foo?bar") );
 #endif
@@ -856,6 +909,10 @@ void KUrlTest::testRelativeURL()
 
   baseURL = "http://www.kde.org/info/index.html";
   QCOMPARE( KUrl::relativeUrl(baseURL, KUrl("http://www.kde.org/bugs/contact.html") ), QString( "../bugs/contact.html") );
+
+  baseURL = "ftp://ftp.kde.org/info/";
+  QCOMPARE( KUrl::relativeUrl(baseURL, KUrl("ftp://ftp.kde.org/info/contact.html") ), QString( "contact.html") );
+  QCOMPARE( KUrl::relativeUrl(baseURL, KUrl("ftp://ftp.kde.org/info/A%20%20B.txt") ), QString( "A%20%20B.txt") );
 }
 
 void KUrlTest::testAdjustPath()
@@ -889,6 +946,15 @@ void KUrlTest::testAdjustPath()
     ftpurl1.adjustPath(KUrl::RemoveTrailingSlash);
     QCOMPARE(  ftpurl1.path(), QString("/" ) );
 
+    KUrl ftpurlNoPath("ftp://ftp.kde.org"); // #312060
+    QCOMPARE(ftpurlNoPath.path(), QString());
+    ftpurlNoPath.adjustPath(KUrl::LeaveTrailingSlash);
+    QCOMPARE(ftpurlNoPath.path(), QString());
+    ftpurlNoPath.adjustPath(KUrl::AddTrailingSlash);
+    QCOMPARE(ftpurlNoPath.path(), QString());
+    ftpurlNoPath.adjustPath(KUrl::RemoveTrailingSlash);
+    QCOMPARE(ftpurlNoPath.path(), QString());
+
     KUrl ftpurl2("ftp://ftp.kde.org///");
     ftpurl2.adjustPath(KUrl::LeaveTrailingSlash);
     QCOMPARE(  ftpurl2.path(), QString("///" ) );
@@ -924,9 +990,13 @@ void KUrlTest::testAdjustPath()
 
     {
     KUrl remote2("remote://");
-    QCOMPARE( remote2.url(), QString("remote:") ); // QUrl bug, fixed in Qt5
-    QCOMPARE( remote2.prettyUrl(), QString("remote://") );
+#if QT_VERSION < 0x040805
+    QCOMPARE( remote2.url(), QString("remote:") ); // QUrl bug, fixed in Qt 4.8.5 and Qt5
+#else
+    QCOMPARE( remote2.url(), QString("remote://") );
+#endif
     QCOMPARE( remote2.url(KUrl::RemoveTrailingSlash ), QString("remote:") ); // QUrl bug, fixed in Qt5
+    QCOMPARE( remote2.prettyUrl(), QString("remote://") );
     }
 }
 
@@ -1082,8 +1152,8 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
      QCOMPARE( waba2.url(), QString("http://www.website.com/directory/?hello#ref") );
   }
   {
-      KUrl waba2( waba1, "#%72%22method"); // #243217
-     QCOMPARE( waba2.url(), QString("http://www.website.com/directory/?hello#%72%22method") );
+     KUrl waba2( waba1, "#%72%22method"); // #243217
+     QCOMPARE( waba2.url(), QString("http://www.website.com/directory/?hello#r%22method") );
   }
   {
      KUrl base( "http://faure@www.kde.org" ); // no path
@@ -1718,7 +1788,11 @@ void KUrlTest::testSmb()
   QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb:/"));
   smb = "smb://"; // KDE3: kurl.cpp rev 1.106 made it invalid. Valid again with QUrl.
   QVERIFY( smb.isValid() );
-  QCOMPARE(smb.url(), QString::fromLatin1("smb:")); // QUrl bug, fixed in Qt5
+#if QT_VERSION < 0x040805
+  QCOMPARE(smb.url(), QString::fromLatin1("smb:")); // QUrl bug, fixed in Qt 4.8.5 and Qt5
+#else
+  QCOMPARE(smb.url(), QString::fromLatin1("smb://"));
+#endif
   QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb://"));
   smb = "smb://host";
   QVERIFY( smb.isValid() );
@@ -1726,7 +1800,11 @@ void KUrlTest::testSmb()
   QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb://host"));
   smb = "smb:///";
   QVERIFY( smb.isValid() );
-  QCOMPARE(smb.url(), QString::fromLatin1("smb:/")); // QUrl bug, fixed in Qt5
+#if QT_VERSION < 0x040805
+  QCOMPARE(smb.url(), QString::fromLatin1("smb:/")); // QUrl bug, fixed in Qt 4.8.5 and Qt5
+#else
+  QCOMPARE(smb.url(), QString::fromLatin1("smb:///"));
+#endif
   QCOMPARE(smb.prettyUrl(), QString::fromLatin1("smb:/"));
 
   KUrl implicitSmb("file://host/path");
